package node
 
func (this *EffectNode) UseSkill() bool {
	panic("not implemented") // TODO: Implement
}
func (this *EffectNode) OnSkillPP() bool {
	panic("not implemented") // TODO: Implement
}
func (this *EffectNode) SkillUseEnd() bool {
	panic("not implemented") // TODO: Implement
}

func (this *EffectNode) BeforeMultiHit() bool {
	panic("not implemented") // TODO: Implement
}

func (this *EffectNode) BeforeHit() bool {
	panic("not implemented") // TODO: Implement
}

func (this *EffectNode) OnCritPreDamage() bool {
	panic("not implemented") // TODO: Implement
}

func (this *EffectNode) PreDamage() bool {
	panic("not implemented") // TODO: Implement
}

func (this *EffectNode) OnBeforeCalculateDamage() bool {
	panic("not implemented") // TODO: Implement
}

func (this *EffectNode) OnDamage() bool {
	panic("not implemented") // TODO: Implement
}

func (this *EffectNode) Shield() bool {
	panic("not implemented") // TODO: Implement
}

func (this *EffectNode) PostDamage() bool {
	panic("not implemented") // TODO: Implement
}

func (this *EffectNode) OnCritPostDamage() bool {
	panic("not implemented") // TODO: Implement
}

func (this *EffectNode) OnHit() bool {
	panic("not implemented") // TODO: Implement
}

func (this *EffectNode) OnMiss() bool {
	panic("not implemented") // TODO: Implement
}

func (this *EffectNode) AfterAttacked() bool {
	panic("not implemented") // TODO: Implement
}

func (this *EffectNode) OnDefeat() bool {
	panic("not implemented") // TODO: Implement
}
